/**
 * Define api of apeman-demo-api
 * @function define
 * @param {boolean} [options.logging=false] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')
const spec = require('./spec')

/** @lends define */
function define (options = {}) {
  let {
    logging = true
  } = options
  let logger = aslogger({ disabled: !logging })

  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },

    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
